package sun.nio.fs;

import java.nio.file.attribute.UserPrincipalNotFoundException;
import java.nio.file.attribute.GroupPrincipal;
import java.nio.file.attribute.UserPrincipal;
import java.util.NoSuchElementException;
import java.util.HashSet;
import java.util.Arrays;
import java.nio.file.WatchService;
import java.util.regex.Pattern;
import java.nio.file.PathMatcher;
import java.nio.file.attribute.UserPrincipalLookupService;
import java.util.Iterator;
import java.nio.file.FileStore;
import java.util.Collections;
import java.util.ArrayList;
import java.nio.file.Path;
import java.io.IOException;
import java.nio.file.spi.FileSystemProvider;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.util.Set;
import java.nio.file.FileSystem;

class WindowsFileSystem extends FileSystem
{
    private final WindowsFileSystemProvider provider;
    private final String defaultDirectory;
    private final String defaultRoot;
    private final boolean supportsLinks;
    private final boolean supportsStreamEnumeration;
    private static final Set<String> supportedFileAttributeViews;
    private static final String GLOB_SYNTAX = "glob";
    private static final String REGEX_SYNTAX = "regex";
    
    WindowsFileSystem(WindowsFileSystemProvider provider, String s) {
        this.provider = provider;
        WindowsPathParser.Result parse = WindowsPathParser.parse(s);
        if (parse.type() != WindowsPathType.ABSOLUTE && parse.type() != WindowsPathType.UNC) {
            throw new AssertionError("Default directory is not an absolute path");
        }
        this.defaultDirectory = parse.path();
        this.defaultRoot = parse.root();
        String[] split = Util.split((String)AccessController.doPrivileged(new GetPropertyAction("os.version")), '.');
        int int1 = Integer.parseInt(split[0]);
        int int2 = Integer.parseInt(split[1]);
        this.supportsLinks = (int1 >= 6);
        this.supportsStreamEnumeration = (int1 >= 6 || (int1 == 5 && int2 >= 2));
        return;
    }
    
    String defaultDirectory() {
        return this.defaultDirectory;
    }
    
    String defaultRoot() {
        return this.defaultRoot;
    }
    
    boolean supportsLinks() {
        return this.supportsLinks;
    }
    
    boolean supportsStreamEnumeration() {
        return this.supportsStreamEnumeration;
    }
    
    @Override
    public FileSystemProvider provider() {
        return this.provider;
    }
    
    @Override
    public String getSeparator() {
        return "\\";
    }
    
    @Override
    public boolean isOpen() {
        return true;
    }
    
    @Override
    public boolean isReadOnly() {
        return false;
    }
    
    @Override
    public void close() throws IOException {
        throw new UnsupportedOperationException();
    }
    
    @Override
    public Iterable<Path> getRootDirectories() {
        int getLogicalDrives;
        try {
            getLogicalDrives = WindowsNativeDispatcher.GetLogicalDrives();
        }
        catch (WindowsException ex) {
            throw new AssertionError(ex.getMessage());
        }
        ArrayList<WindowsPath> list = new ArrayList();
        SecurityManager securityManager = System.getSecurityManager();
        for (int i = 0; i <= 25; ++i) {
            if ((getLogicalDrives & 1 << i) != 0x0) {
                StringBuilder o = new StringBuilder(3);
                o.append((char)(65 + i));
                o.append(":\\");
                String string = o.toString();
                if (securityManager != null) {
                    try {
                        securityManager.checkRead(string);
                    }
                    catch (SecurityException ex2) {
                        continue;
                    }
                }
                list.add(WindowsPath.createFromNormalizedPath(this, string));
            }
        }
        return Collections.unmodifiableList(list);
    }
    
    @Override
    public Iterable<FileStore> getFileStores() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            try {
                securityManager.checkPermission(new RuntimePermission("getFileStoreAttributes"));
            }
            catch (SecurityException ex) {
                return Collections.emptyList();
            }
        }
        return new Iterable<FileStore>() {
            @Override
            public Iterator<FileStore> iterator() {
                return new WindowsFileSystem.FileStoreIterator();
            }
        };
    }
    
    @Override
    public Set<String> supportedFileAttributeViews() {
        return WindowsFileSystem.supportedFileAttributeViews;
    }
    
    @Override
    public final Path getPath(String string, String... array) {
        if (array.length == 0) {
        }
        else {
            StringBuilder sb = new StringBuilder();
            sb.append(string);
            for (String str : array) {
                if (str.length() > 0) {
                    if (sb.length() > 0) {
                        sb.append('\\');
                    }
                    sb.append(str);
                }
            }
            string = sb.toString();
        }
        return WindowsPath.parse(this, string);
    }
    
    @Override
    public UserPrincipalLookupService getUserPrincipalLookupService() {
        return WindowsFileSystem.LookupService.instance;
    }
    
    @Override
    public PathMatcher getPathMatcher(String s) {
        int index = s.indexOf(58);
        if (index <= 0 || index == s.length()) {
            throw new IllegalArgumentException();
        }
        String substring = s.substring(0, index);
        String substring2 = s.substring(index + 1);
        String windowsRegexPattern;
        if (substring.equals("glob")) {
            windowsRegexPattern = Globs.toWindowsRegexPattern(substring2);
        }
        else if (substring.equals("regex")) {
            windowsRegexPattern = substring2;
        }
        else {
            throw new UnsupportedOperationException("Syntax '" + substring + "' not recognized");
        }
        return new PathMatcher() {
            
            @Override
            public boolean matches(Path path) {
                return pattern.matcher(path.toString()).matches();
            }
        };
    }
    
    @Override
    public WatchService newWatchService() throws IOException {
        return new WindowsWatchService(this);
    }
    
    static {
        supportedFileAttributeViews = Collections.unmodifiableSet(new HashSet(Arrays.asList(new String[] { "basic", "dos", "acl", "owner", "user" })));
    }
    
    private class FileStoreIterator implements Iterator<FileStore>
    {
        private final Iterator<Path> roots;
        private FileStore next;
        
        FileStoreIterator() {
            this.roots = WindowsFileSystem.this.getRootDirectories().iterator();
        }
        
        private FileStore readNext() {
            assert !(!Thread.holdsLock(this));
            while (this.roots.hasNext()) {
                WindowsPath windowsPath = (WindowsPath)this.roots.next();
                try {
                    windowsPath.checkRead();
                }
                catch (SecurityException ex) {
                    continue;
                }
                try {
                    WindowsFileStore create = WindowsFileStore.create(windowsPath.toString(), true);
                    if (create != null) {
                        return create;
                    }
                    continue;
                }
                catch (IOException ex2) {}
            }
            return null;
        }
        
        @Override
        public synchronized boolean hasNext() {
            if (this.next != null) {
                return true;
            }
            this.next = this.readNext();
            return this.next != null;
        }
        
        @Override
        public synchronized FileStore next() {
            if (this.next == null) {
                this.next = this.readNext();
            }
            if (this.next == null) {
                throw new NoSuchElementException();
            }
            FileStore next = this.next;
            this.next = null;
            return next;
        }
        
        @Override
        public void remove() {
            throw new UnsupportedOperationException();
        }
    }
    
    private static class LookupService
    {
        static final UserPrincipalLookupService instance;
        
        static {
            instance = new UserPrincipalLookupService() {
                @Override
                public UserPrincipal lookupPrincipalByName(String s) throws IOException {
                    return WindowsUserPrincipals.lookup(s);
                }
                
                @Override
                public GroupPrincipal lookupPrincipalByGroupName(String name) throws IOException {
                    UserPrincipal lookup = WindowsUserPrincipals.lookup(name);
                    if (!(lookup instanceof GroupPrincipal)) {
                        throw new UserPrincipalNotFoundException(name);
                    }
                    return (GroupPrincipal)lookup;
                }
            };
        }
    }
}
